// Neural Space Rev. 2002
// By Hector Lopez
// 10/3/2002

#include "ConfigWindow.h"


RTColorControl::RTColorControl(BPoint point, BMessage *message)
	: BColorControl(point, B_CELLS_32x8, 6, "ColorControl", message, false) {

};


void RTColorControl::SetValue(int32 color) {
	BColorControl::SetValue(color);
	Invoke();
};


ConfigWindow::ConfigWindow(BRect frame, const char *title, passing_things data) :
	BWindow(frame, title, B_TITLED_WINDOW, B_NOT_ZOOMABLE | B_NOT_RESIZABLE | B_ASYNCHRONOUS_CONTROLS) {
	
	active_color = (data.active.red << 24) + (data.active.green << 16) + (data.active.blue << 8);
	idle_color = (data.idle.red << 24) + (data.idle.green << 16) + (data.idle.blue << 8);
	frame_color = (data.frame.red << 24) + (data.frame.green << 16) + (data.frame.blue << 8);
	icon_width = data.width;
	messenger = data.msgTarget;
	
	BRect r(0, 0, 291, 131);
	BBox *bbox = new BBox(r, "BBox");
	bbox->SetLabel("Bar Colors");
	AddChild(bbox);
	
	colorControl = new RTColorControl(BPoint(10, 20), new BMessage(NS_COLOR_CHANGE));
	
	bbox->AddChild(colorControl);
	r = colorControl->Frame();
	r.top = r.bottom + 10;
	r.bottom = r.top + 15;
	
	colorControl->SetValue(active_color);
	
	r.right = r.left + be_plain_font->StringWidth("Active color") + 20;
	activeRadioButton = new BRadioButton(r, "ActiveColor", "Active color",
		new BMessage(NS_ACTIVE_COLOR_SELECTED));
	bbox->AddChild(activeRadioButton);
	activeRadioButton->SetValue(B_CONTROL_ON);
		
	r.left = r.right + 5;
	r.right = r.left + be_plain_font->StringWidth("Idle color") + 20;
	idleRadioButton = new BRadioButton(r, "IdleColor", "Idle color",
		new BMessage(NS_IDLE_COLOR_SELECTED));
	bbox->AddChild(idleRadioButton);
		
	r.left = r.right + 5;
	r.right = r.left + be_plain_font->StringWidth("Frame color") + 20;
	frameRadioButton = new BRadioButton(r, "FrameColor", "Frame color",
		new BMessage(NS_FRAME_COLOR_SELECTED));
	bbox->AddChild(frameRadioButton);
		
	r.top = activeRadioButton->Frame().bottom + 1;
	r.bottom = r.top + 15;
	r.left = 10;
	r.right = r.left + be_plain_font->StringWidth("Width of icon:") + 5 + 30;
	char temp[10];
	sprintf(temp, "%d", icon_width + 1);
	iconWidth = new BTextControl(r, "Width", "Width of icon:", temp,
		new BMessage(NS_ICON_WIDTH_CONTROL));
	bbox->AddChild(iconWidth);
	iconWidth->SetDivider(be_plain_font->StringWidth("Width of icon:") + 5);
	
	for (int x = 0; x < 256; x++) {
		if (x < '0' || x > '9') iconWidth->TextView()->DisallowChar(x);
	}
	iconWidth->TextView()->SetMaxBytes(2);
	
	r.left = r.right + 35;
	r.right = r.left + be_plain_font->StringWidth("Reset To Defaults") + 30;
	r.top = r.top - 3;
	r.bottom = r.bottom - 6;
	resetButton = new BButton(r, "ResetButton", "Reset To Defaults", new BMessage(NS_DEFAULTS));
	bbox->AddChild(resetButton);
};

void ConfigWindow::MessageReceived(BMessage *msg) {	
	
	switch (msg->what) {
		case NS_ACTIVE_COLOR_SELECTED: {
			colorControl->BColorControl::SetValue(active_color);
			break;
		};
		case NS_IDLE_COLOR_SELECTED: {
			colorControl->BColorControl::SetValue(idle_color);
			break;
		};
		case NS_FRAME_COLOR_SELECTED: {
			colorControl->BColorControl::SetValue(frame_color);
			break;
		};
		case NS_ICON_WIDTH_CONTROL: {
			UpdateDeskbarIconWidth();
			break;
		};
		case NS_COLOR_CHANGE: {
			BMessage *update = new BMessage(NS_COLOR_CHANGE);
			if (activeRadioButton->Value() > 0) {
				active_color = colorControl->Value();
				update->AddInt32("active_color", active_color);
			} else {
				if (idleRadioButton->Value() > 0) {
					idle_color = colorControl->Value();
					update->AddInt32("idle_color", idle_color);
				} else {
					frame_color = colorControl->Value();
					update->AddInt32("frame_color", frame_color);
				};
			};
			messenger->SendMessage(update);
			delete update;
			break;
		};
		case NS_DEFAULTS:
			ResetToDefaults();
			break;
		default:
			BWindow::MessageReceived(msg);
	};
};

void ConfigWindow::UpdateDeskbarIconWidth() {
	int width = atoi(iconWidth->Text());
	if (width < 3 || width > 50) {
		char temp[10];
		if (width < 3) {
			sprintf(temp, "%d", 3);
			width = 3;
		} else {
			strcpy(temp, "50");
			width = 50;
		}
		iconWidth->SetText(temp);
	}
	
	BMessage *message = new BMessage(NS_ICON_WIDTH_CONTROL);
	message->AddInt8("width", width - 1);
	messenger->SendMessage(message);
	delete message;
};

void ConfigWindow::ResetToDefaults() {
	active_color = NS_DEFAULT_ACTIVE_COLOR;
	idle_color = NS_DEFAULT_IDLE_COLOR;
	frame_color = NS_DEFAULT_FRAME_COLOR;
	
	colorControl->BColorControl::SetValue(active_color);
	activeRadioButton->SetValue(B_CONTROL_ON);
	
	icon_width = 8;
	iconWidth->SetText("9");
	
	BMessage *reset = new BMessage(NS_RESET);
	messenger->SendMessage(reset);
	delete reset;
};

bool ConfigWindow::QuitRequested() {
	BMessage *message = new BMessage(NS_PREFS_WINDOW_DIED);
	messenger->SendMessage(message);
	delete message;
	return true;
};


	